@@ -19,9 +19,12 @@ from flox import xrutils
 from flox.aggregations import Aggregation, _initialize_aggregation
 from flox.core import (
     HAS_NUMBAGG,
+    ReindexArrayType,
+    ReindexStrategy,
     _choose_engine,
     _convert_expected_groups_to_index,
     _get_optimal_chunks_for_groups,
+    _is_sparse_supported_reduction,
     _normalize_indexes,
     _validate_reindex,
     factorize_,
@@ -41,9 +44,11 @@ from . import (
     assert_equal_tuple,
     has_cubed,
     has_dask,
+    has_sparse,
     raise_if_dask_computes,
     requires_cubed,
     requires_dask,
+    requires_sparse,
 )
 
 logger = logging.getLogger("flox")
@@ -71,6 +76,10 @@ if has_cubed:
 
 
 DEFAULT_QUANTILE = 0.9
+REINDEX_SPARSE_STRAT = ReindexStrategy(blockwise=False, array_type=ReindexArrayType.SPARSE_COO)
+REINDEX_SPARSE_PARAM = pytest.param(
+    REINDEX_SPARSE_STRAT, marks=(requires_dask, pytest.mark.skipif(not has_sparse, reason="no sparse"))
+)
 
 if TYPE_CHECKING:
     from flox.core import T_Agg, T_Engine, T_ExpectedGroupsOpt, T_Method
@@ -317,13 +326,20 @@ def test_groupby_reduce_all(nby, size, chunks, func, add_nan_by, engine):
         if not has_dask or chunks is None or func in BLOCKWISE_FUNCS:
             continue
 
-        params = list(itertools.product(["map-reduce"], [True, False, None]))
+        params = list(
+            itertools.product(
+                ["map-reduce"],
+                [True, False, None, REINDEX_SPARSE_STRAT],
+            )
+        )
         params.extend(itertools.product(["cohorts"], [False, None]))
         if chunks == -1:
             params.extend([("blockwise", None)])
 
         combine_error = RuntimeError("This combine should not have been called.")
         for method, reindex in params:
+            if isinstance(reindex, ReindexStrategy) and not _is_sparse_supported_reduction(func):
+                continue
             call = partial(
                 groupby_reduce,
                 array,
@@ -357,6 +373,10 @@ def test_groupby_reduce_all(nby, size, chunks, func, add_nan_by, engine):
                 assert_equal(actual_group, expect, tolerance)
             if "arg" in func:
                 assert actual.dtype.kind == "i"
+            if isinstance(reindex, ReindexStrategy):
+                import sparse
+
+                expected = sparse.COO.from_numpy(expected)
             assert_equal(actual, expected, tolerance)
 
 
@@ -444,7 +464,7 @@ def test_numpy_reduce_nd_md():
 
 
 @requires_dask
-@pytest.mark.parametrize("reindex", [None, False, True])
+@pytest.mark.parametrize("reindex", [None, False, True, REINDEX_SPARSE_PARAM])
 @pytest.mark.parametrize("func", ALL_FUNCS)
 @pytest.mark.parametrize("add_nan", [False, True])
 @pytest.mark.parametrize("dtype", (float,))
@@ -467,6 +487,9 @@ def test_groupby_agg_dask(func, shape, array_chunks, group_chunks, add_nan, dtyp
     if "arg" in func and (engine in ["flox", "numbagg"] or reindex):
         pytest.skip()
 
+    if isinstance(reindex, ReindexStrategy) and not _is_sparse_supported_reduction(func):
+        pytest.skip()
+
     rng = np.random.default_rng(12345)
     array = dask.array.from_array(rng.random(shape), chunks=array_chunks).astype(dtype)
     array = dask.array.ones(shape, chunks=array_chunks)
@@ -772,6 +795,7 @@ def test_groupby_reduce_axis_subset_against_numpy(func, axis, engine):
         (None, None),
         pytest.param(False, (2, 2, 3), marks=requires_dask),
         pytest.param(True, (2, 2, 3), marks=requires_dask),
+        pytest.param(REINDEX_SPARSE_PARAM, (2, 2, 3), marks=requires_dask),
     ],
 )
 @pytest.mark.parametrize(
@@ -818,7 +842,13 @@ def test_groupby_reduce_nans(reindex, chunks, axis, groups, expected_shape, engi
 @requires_dask
 @pytest.mark.parametrize(
     "expected_groups, reindex",
-    [(None, None), (None, False), ([0, 1, 2], True), ([0, 1, 2], False)],
+    [
+        (None, None),
+        (None, False),
+        ([0, 1, 2], True),
+        ([0, 1, 2], False),
+        pytest.param([0, 1, 2], REINDEX_SPARSE_PARAM),
+    ],
 )
 def test_groupby_all_nan_blocks_dask(expected_groups, reindex, engine):
     labels = np.array([0, 0, 2, 2, 2, 1, 1, 2, 2, 1, 1, 0])
@@ -1637,7 +1667,7 @@ def test_validate_reindex_map_reduce(dask_expected, reindex, func, expected_grou
         is_dask_array=True,
         array_dtype=np.dtype("int32"),
     )
-    assert actual is dask_expected
+    assert actual == ReindexStrategy(blockwise=dask_expected)
 
     # always reindex with all numpy inputs
     actual = _validate_reindex(
@@ -1649,7 +1679,7 @@ def test_validate_reindex_map_reduce(dask_expected, reindex, func, expected_grou
         is_dask_array=False,
         array_dtype=np.dtype("int32"),
     )
-    assert actual
+    assert actual.blockwise
 
     actual = _validate_reindex(
         True,
@@ -1660,7 +1690,7 @@ def test_validate_reindex_map_reduce(dask_expected, reindex, func, expected_grou
         is_dask_array=False,
         array_dtype=np.dtype("int32"),
     )
-    assert actual
+    assert actual.blockwise
 
 
 def test_validate_reindex() -> None:
@@ -1699,7 +1729,7 @@ def test_validate_reindex() -> None:
                 any_by_dask=False,
                 is_dask_array=True,
                 array_dtype=np.dtype("int32"),
-            )
+            ).blockwise
             assert actual is False
 
     with pytest.raises(ValueError):
@@ -1721,7 +1751,7 @@ def test_validate_reindex() -> None:
         any_by_dask=True,
         is_dask_array=True,
         array_dtype=np.dtype("int32"),
-    )
+    ).blockwise
     assert _validate_reindex(
         None,
         "sum",
@@ -1730,7 +1760,7 @@ def test_validate_reindex() -> None:
         any_by_dask=True,
         is_dask_array=True,
         array_dtype=np.dtype("int32"),
-    )
+    ).blockwise
 
     kwargs = dict(
         method="blockwise",
@@ -1740,12 +1770,12 @@ def test_validate_reindex() -> None:
     )
 
     for func in ["nanfirst", "nanlast"]:
-        assert not _validate_reindex(None, func, array_dtype=np.dtype("int32"), **kwargs)  # type: ignore[arg-type]
-        assert _validate_reindex(None, func, array_dtype=np.dtype("float32"), **kwargs)  # type: ignore[arg-type]
+        assert not _validate_reindex(None, func, array_dtype=np.dtype("int32"), **kwargs).blockwise  # type: ignore[arg-type]
+        assert _validate_reindex(None, func, array_dtype=np.dtype("float32"), **kwargs).blockwise  # type: ignore[arg-type]
 
     for func in ["first", "last"]:
-        assert not _validate_reindex(None, func, array_dtype=np.dtype("int32"), **kwargs)  # type: ignore[arg-type]
-        assert not _validate_reindex(None, func, array_dtype=np.dtype("float32"), **kwargs)  # type: ignore[arg-type]
+        assert not _validate_reindex(None, func, array_dtype=np.dtype("int32"), **kwargs).blockwise  # type: ignore[arg-type]
+        assert not _validate_reindex(None, func, array_dtype=np.dtype("float32"), **kwargs).blockwise  # type: ignore[arg-type]
 
 
 @requires_dask
@@ -2023,20 +2053,18 @@ def test_datetime_minmax(engine) -> None:
 
 @pytest.mark.parametrize("func", ["first", "last", "nanfirst", "nanlast"])
 def test_datetime_timedelta_first_last(engine, func) -> None:
-    import flox
-
     idx = 0 if "first" in func else -1
     idx1 = 2 if "first" in func else -1
 
     ## datetime
     dt = pd.date_range("2001-01-01", freq="d", periods=5).values
     by = np.ones(dt.shape, dtype=int)
-    actual, *_ = flox.groupby_reduce(dt, by, func=func, engine=engine)
+    actual, *_ = groupby_reduce(dt, by, func=func, engine=engine)
     assert_equal(actual, dt[[idx]])
 
     # missing group
     by = np.array([0, 2, 3, 3, 3])
-    actual, *_ = flox.groupby_reduce(
+    actual, *_ = groupby_reduce(
         dt, by, expected_groups=([0, 1, 2, 3],), func=func, engine=engine, fill_value=dtypes.NA
     )
     assert_equal(actual, [dt[0], np.datetime64("NaT"), dt[1], dt[idx1]])
@@ -2044,12 +2072,79 @@ def test_datetime_timedelta_first_last(engine, func) -> None:
     ## timedelta
     dt = dt - dt[0]
     by = np.ones(dt.shape, dtype=int)
-    actual, *_ = flox.groupby_reduce(dt, by, func=func, engine=engine)
+    actual, *_ = groupby_reduce(dt, by, func=func, engine=engine)
     assert_equal(actual, dt[[idx]])
 
     # missing group
     by = np.array([0, 2, 3, 3, 3])
-    actual, *_ = flox.groupby_reduce(
+    actual, *_ = groupby_reduce(
         dt, by, expected_groups=([0, 1, 2, 3],), func=func, engine=engine, fill_value=dtypes.NA
     )
     assert_equal(actual, [dt[0], np.timedelta64("NaT"), dt[1], dt[idx1]])
+
+
+@requires_dask
+@requires_sparse
+@pytest.mark.xdist_group(name="sparse-group")
+@pytest.mark.parametrize("size", [2**62 - 1, 11])
+def test_reindex_sparse(size):
+    import sparse
+
+    array = dask.array.ones((2, 12), chunks=(-1, 3))
+    func = "sum"
+    expected_groups = pd.RangeIndex(size)
+    by = dask.array.from_array(np.repeat(np.arange(6) * 2, 2), chunks=(3,))
+    dense = np.zeros((2, 11))
+    dense[..., np.arange(6) * 2] = 2
+    expected = sparse.COO.from_numpy(dense)
+
+    with pytest.raises(ValueError):
+        ReindexStrategy(blockwise=True, array_type=ReindexArrayType.SPARSE_COO)
+    reindex = ReindexStrategy(blockwise=False, array_type=ReindexArrayType.SPARSE_COO)
+
+    original_reindex = flox.core.reindex_
+
+    def mocked_reindex(*args, **kwargs):
+        res = original_reindex(*args, **kwargs)
+        if isinstance(res, dask.array.Array):
+            assert isinstance(res._meta, sparse.COO)
+        else:
+            assert isinstance(res, sparse.COO)
+        return res
+
+    # Define the error-raising property
+    def raise_error(self):
+        raise AttributeError("Access to '_data' is not allowed.")
+
+    with patch("flox.core.reindex_") as mocked_reindex_func:
+        with patch.object(pd.RangeIndex, "_data", property(raise_error)):
+            mocked_reindex_func.side_effect = mocked_reindex
+            actual, *_ = groupby_reduce(
+                array, by, func=func, reindex=reindex, expected_groups=expected_groups, fill_value=0
+            )
+            if size == 11:
+                assert_equal(actual, expected)
+            else:
+                actual.compute()  # just compute
+
+            # once during graph construction, 10 times afterward
+            assert mocked_reindex_func.call_count > 1
+
+
+def test_sparse_errors():
+    call = partial(
+        groupby_reduce,
+        [1, 2, 3],
+        [0, 1, 1],
+        reindex=REINDEX_SPARSE_STRAT,
+        fill_value=0,
+        expected_groups=[0, 1, 2],
+    )
+
+    if not has_sparse:
+        with pytest.raises(ImportError):
+            call(func="sum")
+
+    else:
+        with pytest.raises(ValueError):
+            call(func="first")
